__version__ = '0.1'


# needed for inheritance and adding in folders
from OFS.Folder import Folder
from OFS.Image import Image
from OFS.DTMLMethod import DTMLMethod

# usual cut and paste imports
from Globals import HTMLFile
from Globals import MessageDialog
from Persistence import Persistent

import Acquisition
import OFS.SimpleItem
import AccessControl.Role

# usual imports
import string
import os
import Globals
import sys
import time # used for file object ids

# dtml files for adding forms
manage_addFSPollForm = HTMLFile('FSPollAdd', globals())
manage_addFSPollQuestionForm = HTMLFile('FSPollQuestionAdd', globals())
manage_addFSPollAnswerForm = HTMLFile('FSPollAnswerAdd', globals())

def manage_addFSPoll(self, id, title='', REQUEST=None):
    ''' Adds a new FSPoll object '''
    obj = FSPoll() # standard folder constructor
    obj.id = id
    obj.title = title

    self._setObject(id, obj)
    obj.initialise() # folder creation stuff
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

def manage_addFSPollQuestion(self, id, title='', REQUEST=None):
    ''' Adds a new object '''
    obj = FSPollQuestion() # standard folder constructor
    obj.id = id
    obj.title = title

    self._setObject(id, obj)
    obj.initialise() # folder creations stuff
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

def manage_addFSPollAnswer(self, title='', REQUEST=None):
    ''' Adds a new object '''
    id = str(self.get_answer_number()) # finds the id for next answer
    obj = FSPollAnswer(id, title) # this isnt a folder, I'll happily make an __init__ and call it normally
    self._setObject(id, obj)
    obj.initialise() # object creation stuff
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)

# FSPoll is a container for the rest of the poll so that
# the HTML methods will be visible to users so they can edit it.
class FSPoll(Folder):
    meta_type = 'FSPoll'
    isPrincipiaFolderish = 1

    __ac_permissions__=( ('View management screens', ('initialise',) ), )

    def filtered_meta_types(self):
        '''
        returns the objects we want from the tuple
        so people can only add in Answers into a Question
        '''
        l = []
        types = Folder.filtered_meta_types(self)
        for t in types:
            if t['name'] not in ['FSPollAnswer']:
                l.append(t)
        return l

    def _read_file(self, filename):
        ''' reads file from filesystem '''
        file = open(filename, 'rb')
        data = file.read()
        file.close()
        return data

    def _put_file_in(self, file):
        ''' reads then puts file in ZODB '''
        data = self._read_file(Globals.package_home(globals()) + os.sep + file)
        obj = DTMLMethod(data, __name__ = file)
        self._setObject(file, obj)

    def _put_image_in(self, file):
        ''' reads then puts image in ZODB '''
        data = self._read_file(Globals.package_home(globals()) + os.sep + file)
        obj = Image(id=file, title='Yes', file=data)
        self._setObject(file, obj)

    def initialise(self):
        ''' adds in search and index page '''
        for file in ['index_html', 'vote_big_html', 'vote_small_html', 'results_html']:
            self._put_file_in(file)
        for image in ['yes.gif']:
            self._put_image_in(image)
        self.root = string.join(self.getPhysicalPath(), '/')

        # try to make directories for file objects
        try: os.makedirs(Globals.data_dir + os.sep + 'FSPoll')
        except OSError: pass # means file already exists

# FSPollQuestion is the object for the poll question, its a container
# so it can have the answers in it.
class FSPollQuestion(Folder):
    meta_type = 'FSPollQuestion'
    isPrincipiaFolderish = 1

    # copy and paste would screw up the file system, if people had two answers
    # pointing to the same file on the FS
    dontAllowCopyAndPaste = 1
    property_extensible_schema__ = 0

    __ac_permissions__=(
        ('View management screens', ('initialise', 'filtered_meta_types', 'manage_edit', 'get_answer_number', 'manage_editForm') ),
        ('Vote', ('add_vote', ) ),
        ('Access contents information', ('total_votes', ) ),
    )

    # weird Zope 2.3 manage_options hack
    _m = list(Folder.manage_options)
    _m[1] = {'label':'View','action':'vote_big_html'}
    _m[2] = {'label':'Properties','action':'manage_editForm'}
    manage_options = (_m)

    # edit form
    manage_editForm = HTMLFile('FSPollQuestionEdit', globals())

    def filtered_meta_types(self):
        '''
        returns the objects we want from the tuple
        so people can only add in Answers into a Question
        '''
        l = []
        types = Folder.filtered_meta_types(self)
        for t in types:
            if t['name'] in ['FSPollAnswer']:
                l.append(t)
        return l

    def initialise(self):
        ''' start it up '''
        self.open = 1

    def manage_edit(self, title, open=0, REQUEST=None):
        ''' edits the poll '''
        self.title=title
        self.open=open
        if REQUEST is not None:
            return MessageDialog( title='Edited', message='<strong>%s</strong> has been edited.' % self.id, action ='./manage_main', )

    def get_answer_number(self):
        ''' gets an id for the next answer '''
        k = 0
        n = "Answer%s" % k
        for i in self.objectIds(['FSPollAnswer']):
            k = k + 1
            n = "Answer%s" % k
        return n

    def add_vote(self, answer='', REQUEST=None):
        ''' count vote '''
        if not self.open:
            if REQUEST is not None:
                return MessageDialog(title='Error', message='This poll is not open.', action ='.' )
            else:
                return None

        obj = getattr(self, answer)
        try:
            obj.add_vote()
            return MessageDialog(title='Thanks', message='Vote accepted.', action ='./results_html')
        except:
            return MessageDialog(title='Error', message='Sorry an internal error occured, your vote was not counted.<!-- %s, %s-->' % (sys.exc_type, sys.exc_value), action ='./results_html')

    def total_votes(self):
        ''' get a number of total votes '''
        total = 0
        for answer in self.objectIds(['FSPollAnswer']):
            obj = getattr(self, answer)
            total = total + obj.read()

        return int(total)
    # thats it

# FSPollAnswer is the actual answer, a simple item with data on the file system
class FSPollAnswer(OFS.SimpleItem.Item, Persistent, Acquisition.Implicit, AccessControl.Role.RoleManager):
    meta_type = 'FSPollAnswer'

    # dont need the hack here, wierd...
    manage_options = ( ( {'label':'Edit', 'action':'manage_editForm'}, ) + (OFS.SimpleItem.Item.manage_options))
    manage_editForm = HTMLFile('FSPollAnswerEdit', globals())

    __ac_permissions__=(
        ('View management screens', ('initialise', 'filtered_meta_types', 'manage_edit', 'manage_beforeDelete', 'manage_editForm', 'reset') ),
        ('Vote', ('add_vote', ) ),
        ('Access contents information', ('votes', 'read' ) ),
    )

    def __init__(self, id, title='', REQUEST=None):
        ''' init the object '''
        self.id = id
        self.title = title
        self.filename = ''
        if REQUEST is not None:
            return MessageDialog( title='Added', message='Question has been added.', action ='./manage_main', )

    def initialise(self):
        ''' make a filename for the object based on time '''
        self.filename = Globals.data_dir + os.sep + 'FSPoll' + os.sep + str(self.id + '.' + str(time.time()) + '.fspollanswer')

    def add_vote(self):
        ''' increase '''
        return self._write(count=1)

    def votes(self):
        ''' number of votes '''
        return int(self.read())

    def read(self):
        ''' read file '''
        try:
            infile = open(self.filename, 'rb')
            value = int(infile.read())
            infile.close()
        except IOError:
            value = 0 # error reading file
        return value

    def manage_edit(self, title, reset=0, REQUEST=None):
        ''' edits the answer '''
        self.title = title
        if REQUEST is not None:
            return MessageDialog(title = 'Edited', message = "Answer edited", action = './manage_main',)

    def manage_beforeDelete(self, item, container):
        ''' clean up '''
        self._delete()

    def reset(self, REQUEST=None):
        ''' resets the question by deleting the values '''
        if self._delete() is 0:
            if REQUEST is not None:
                return MessageDialog(title = "Error", message = "There was an error reseting this answer.", action = './manage_editForm',)
        if REQUEST is not None:
            return MessageDialog(title = 'Edited', message = "Votes reset", action = './manage_editForm',)

    def _delete(self):
        ''' delete the object '''
        try:
            os.unlink(self.filename)
            return 1
        except:
            return 0 # error deleting, uh oh

    def _write(self, count=0):
        ''' increment file '''
        value = self.read()
        outfile = open(self.filename, 'wb')
        outfile.write(str(value + count))
        outfile.close()
        return int(value + count)